#!/usr/bin/perl -- # -*- Perl -*-

use strict;
use Spreadsheet::ParseExcel;

# Version 1.0  GS 17.12.01 -> first Version
# Version 1.1  GS 18.12.01 -> CC Definition or Definition placed as first line in annotation
# Version 1.2  GS 18.12.01 -> Concatenation of "Type" if the specific DataType do not have this required extension.
# Version 1.3  GS 08.08.02 -> Improvements
# Version 1.4  ndw 26 Aug 2002 -> Hacking

my $usage = "Usage: $0 xlsfilename sheetNumbers rootElement(s)\n";
my(@aHeaderName, @aContentName);

my $xlsFile = shift @ARGV;
my @iSheets = ();
my @rootElements = ();

while (@ARGV) {
    my $param = shift @ARGV;
    if ($param =~ /^\d+$/) {
	push (@iSheets, $param);
    } else {
	push (@rootElements, $param);
    }
}

die $usage if !defined($xlsFile) || ! -f $xlsFile;
die $usage if !@iSheets;

my $targetNamespace = "urn:oasis:names:tc:ubl:Order:1.0:0.65";
my $targetPrefix = "po";
my $cctNamespace = "urn:oasis:names:tc:ubl:CommonLeafTypes:1.0:0.65";

print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
print "<!-- 26 Aug 2002 - Version 1.0r0.65 - source: ", $xlsFile, " -->\n";

print "<xsd:schema targetNamespace='$targetNamespace'\n";
foreach my $attr ("xmlns:xsd='http://www.w3.org/2001/XMLSchema'",
		  "xmlns:xhtml='http://www.w3.org/1999/xhtml'",
		  "xmlns:cct='$cctNamespace'",
		  "xmlns:$targetPrefix='$targetNamespace'",
		  "elementFormDefault='qualified'",
		  "attributeFormDefault='unqualified'") {
    print "            $attr\n";
}
print ">\n\n";

print "<xsd:import namespace='$cctNamespace'\n";
print "            schemaLocation='CoreComponentTypes.xsd'/>\n\n";

print STDERR "Loading $xlsFile...\n";
my $oExcel = new Spreadsheet::ParseExcel;
my $oBook = $oExcel->Parse($xlsFile);

foreach my $iSheet (@iSheets) {
    print STDERR "Parsing sheet $iSheet...\n";
    @aHeaderName = ();
    @aContentName = ();;

    PrnBook($oBook, $iSheet);
}

if (@rootElements) {
    foreach my $elem (@rootElements) {
	print STDERR "Outputting top-level element: $targetPrefix:$elem\n";
	print "\t<xsd:element name='$elem' type='$targetPrefix:${elem}Type'/>\n";
    }
} else {
    print STDERR "Warning: No top-level elements defined!\n";
}

printf("</xsd:schema>\n");

sub PrnBook
{
        my($oBook) = shift;
        my($iSheet) = shift;
	my($iRow, $iCol, $oWorkSheet, $oWorkCol);
	my($bComplexFlag, $bGroupFlag);
	my($sHeaderContent, $sCellContent);
	my(@aElementName, @aElementMatrix);
	my($i, $y);
	my($sMinOccur, $sMaxOccur);

	$oWorkSheet = $oBook->{Worksheet}[$iSheet];

	for(my $iRow = $oWorkSheet->{MinRow} ; defined $oWorkSheet->{MaxRow} && $iRow <= $oWorkSheet->{MaxRow} ; $iRow++) {
		for(my $iCol = $oWorkSheet->{MinCol} ; defined $oWorkSheet->{MaxCol} && $iCol <= $oWorkSheet->{MaxCol} ; $iCol++) {

			$oWorkCol = $oWorkSheet->{Cells}[$iRow][$iCol];
			if($iRow == 0) {
				$sHeaderContent = $oWorkCol->Value if($oWorkCol);
				$sHeaderContent =~ s/[:()']//g; # ' ]
				$sHeaderContent =~ s/[\/ ]/_/g;
				push(@aHeaderName, $sHeaderContent);
			}
			else {
				$sCellContent = $oWorkCol->Value if($oWorkCol);

				if($iCol == 0) {
					$bComplexFlag = "YES" if($sCellContent eq "RT" );
					$bGroupFlag = "YES" if($sCellContent eq "XX" );
				}
				if($bComplexFlag eq "YES") {
					push(@aContentName, $sCellContent);
					$sCellContent = "";
				}
				else {
					push(@aElementName, $sCellContent);
					$sCellContent = "";
				}

			}
		}

		if($bComplexFlag ne "YES") {
			push(@aElementMatrix, [ @aElementName ] );
			@aElementName = ();
		}

		$bComplexFlag = "NO";

		if ($bGroupFlag eq "YES") {
			@aContentName[4] =~ s/[., ]//g;
			printf ("\t<xsd:complexType name=\"%sType\" id=\"%s\">\n", @aContentName[4], @aContentName[2]);
			AnnotationOut ();
			printf ("\t\t<xsd:sequence>\n");
			for ($i=0; $i<@aElementMatrix; $i++) {
				if ( @aElementMatrix[$i]->[4] ne "" ) {
					@aElementMatrix[$i]->[11] =~ s/[., ]//g;
					@aElementMatrix[$i]->[4] =~ s/ //g;
					$sMinOccur = @aElementMatrix[$i]->[12];
					$sMinOccur =~ s/\.\..*//g;
					$sMaxOccur = @aElementMatrix[$i]->[12];
					$sMaxOccur =~ s/.*\.\.//g;
					$sMaxOccur = "unbounded" if ($sMaxOccur eq "n");
					if (@aElementMatrix[$i]->[13] eq "Basic") {
						printf ("\t\t\t<xsd:element name=\"%s\" type=\"cct:%sType\" id=\"%s\" minOccurs=\"%s\" maxOccurs=\"%s\">\n", @aElementMatrix[$i]->[4], @aElementMatrix[$i]->[10], @aElementMatrix[$i]->[2], $sMinOccur, $sMaxOccur);
					}
					else {
						printf ("\t\t\t<xsd:element name=\"%s\" type=\"$targetPrefix:%sType\" id=\"%s\" minOccurs=\"%s\" maxOccurs=\"%s\">\n", @aElementMatrix[$i]->[4], @aElementMatrix[$i]->[10], @aElementMatrix[$i]->[2], $sMinOccur, $sMaxOccur);
					}
					for ($y=0; $y<@aHeaderName; $y++) {
						@aContentName[$y] = @aElementMatrix[$i]->[$y];
					}
					AnnotationOut($bGroupFlag);
					printf ("\t\t\t</xsd:element>\n");
				}
			}
			printf ("\t\t</xsd:sequence>\n");
			printf ("\t</xsd:complexType>\n\n");
			$bGroupFlag = "NO";
			@aContentName = ();
			@aElementMatrix = ();
		}
	}
}

sub AnnotationOut ()
{
	my $z;
	my @aAnnotationSequence=(14,5,10,11,13,15,16,17,18,19,20,21,22,23,24,25,26);
	printf ("\t\t<xsd:annotation>\n");
	printf("\t\t\t<xsd:documentation>\n");
	for ($z=0; $z<@aAnnotationSequence; $z++) {
		if ( @aContentName[@aAnnotationSequence[$z]] ne "") {
			printf("\t\t\t\t<xhtml:div class=\"%s\">\n", @aHeaderName[@aAnnotationSequence[$z]]);
			printf("\t\t\t\t\t<xhtml:p>%s</xhtml:p>\n", @aContentName[@aAnnotationSequence[$z]]);
			printf("\t\t\t\t</xhtml:div>\n");
		}
	}
	printf("\t\t\t</xsd:documentation>\n");
	printf ("\t\t</xsd:annotation>\n");
}
